import { DelegateEventHandler } from "web-utility";
import { DataObject, VNode, JsxChildren } from "dom-renderer";
import { IReactionPublic } from "mobx";
export interface ComponentMeta extends ElementDefinitionOptions, Partial<ShadowRootInit> {
    tagName: string;
}
export type ComponentClass = CustomElementConstructor;
interface DelegatedEvent {
    type: keyof HTMLElementEventMap;
    selector: string;
    handler: EventListener;
}
/**
 * `class` decorator of Web components
 */
export function component(meta: ComponentMeta): <T extends CustomElementConstructor>(Class: T, { addInitializer }: ClassDecoratorContext<ComponentClass>) => T;
/**
 * Method decorator of DOM Event delegation
 */
export function on<T extends HTMLElement>(type: DelegatedEvent['type'], selector: string): (method: DelegateEventHandler, { addInitializer }: ClassMethodDecoratorContext<T>) => void;
export type PropsWithChildren<P extends DataObject = {}> = P & {
    children?: JsxChildren;
};
export type FunctionComponent<P extends DataObject = {}> = (props: P) => VNode;
export type FC<P extends DataObject = {}> = FunctionComponent<P>;
export type WebCellComponent = FunctionComponent | ComponentClass;
/**
 * `class` decorator of Web components for MobX
 */
export function observer<T extends WebCellComponent>(func: T, _: ClassDecoratorContext): T;
export function observer<T extends WebCellComponent>(func: T): T;
/**
 * `accessor` decorator of MobX `@observable` for HTML attributes
 */
export function attribute<C extends HTMLElement, V>(_: ClassAccessorDecoratorTarget<C, V>, { name, addInitializer }: ClassAccessorDecoratorContext<C>): void;
export type ReactionExpression<I = any, O = any> = (data?: I, reaction?: IReactionPublic) => O;
export type ReactionEffect<V> = (newValue: V, oldValue: V, reaction: IReactionPublic) => any;
/**
 * Method decorator of MobX `reaction()`
 */
export function reaction<C extends HTMLElement, V>(expression: ReactionExpression<C, V>): (effect: ReactionEffect<V>, { addInitializer }: ClassMethodDecoratorContext<C>) => void;
/**
 * `class` decorator of Form associated Web components
 */
export function formField<T extends ComponentClass>(Class: T, _: ClassDecoratorContext): T;

//# sourceMappingURL=index.d.ts.map
